/*
    This file is part of the kolab resource - the implementation of the
    Kolab storage format. See www.kolab.org for documentation on this.

    Copyright (c) 2004 Bo Thorsen <bo@sonofthor.dk>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.

    In addition, as a special exception, the copyright holders give
    permission to link the code of this program with any edition of
    the TQt library by Trolltech AS, Norway (or with modified versions
    of TQt that use the same license as TQt), and distribute linked
    combinations including the two.  You must obey the GNU General
    Public License in all respects for all of the code used other than
    TQt.  If you modify this file, you may extend this exception to
    your version of the file, but you are not obligated to do so.  If
    you do not wish to do so, delete this exception statement from
    your version.
*/

#ifndef KOLAB_TASK_H
#define KOLAB_TASK_H

#include <incidence.h>

#include <libkcal/incidence.h>

class TQDomElement;

namespace KCal {
  class Todo;
  class ResourceKolab;
}

namespace Kolab {

/**
 * This class represents a task, and knows how to load/save it
 * from/to XML, and from/to a KCal::Todo.
 * The instances of this class are temporary, only used to convert
 * one to the other.
 */
class Task : public Incidence {
public:
  /// Use this to parse an xml string to a task entry
  /// The caller is responsible for deleting the returned task
  static KCal::Todo* xmlToTask( const TQString& xml, const TQString& tz, KCal::ResourceKolab *res = 0,
                                const TQString& subResource = TQString(), TQ_UINT32 sernum = 0 );

  /// Use this to get an xml string describing this task entry
  static TQString taskToXML( KCal::Todo*, const TQString& tz );

  explicit Task( KCal::ResourceKolab *res, const TQString& subResource, TQ_UINT32 sernum,
                 const TQString& tz, KCal::Todo* todo = 0 );
  virtual ~Task();

  virtual TQString type() const { return "Task"; }

  void saveTo( KCal::Todo* todo );

  virtual void setPriority( int priority );
  virtual int priority() const;

  virtual void setPercentCompleted( int percent );
  virtual int percentCompleted() const;

  virtual void setStatus( KCal::Incidence::Status status );
  virtual KCal::Incidence::Status status() const;

  virtual void setParent( const TQString& parentUid );
  virtual TQString parent() const;

  virtual void setHasStartDate( bool );
  virtual bool hasStartDate() const;

  virtual void setDueDate( const TQDateTime &date );
  virtual void setDueDate( const TQString &date );
  virtual void setDueDate( const TQDate &date );
  virtual TQDateTime dueDate() const;
  virtual bool hasDueDate() const;

  virtual void setCompletedDate( const TQDateTime& date );
  virtual TQDateTime completedDate() const;
  virtual bool hasCompletedDate() const;

  // Load the attributes of this class
  virtual bool loadAttribute( TQDomElement& );

  // Save the attributes of this class
  virtual bool saveAttributes( TQDomElement& ) const;

  // Load this task by reading the XML file
  virtual bool loadXML( const TQDomDocument& xml );

  // Serialize this task to an XML string
  virtual TQString saveXML() const;

protected:
  // Read all known fields from this ical todo
  void setFields( const KCal::Todo* );

  // This sets the priority of this task by looking at mKolabPriorityFromDom and
  // mKCalPriorityFromDom.
  void decideAndSetPriority();

  // This is the KCal priority, not the Kolab priority.
  // See kcalPriorityToKolab() and kolabPrioritytoKCal().
  int mPriority;

  // Those priority values are the raw values read by loadAttribute().
  // They will be converted later in decideAndSetPriority().
  int mKolabPriorityFromDom;
  int mKCalPriorityFromDom;

  int mPercentCompleted;
  KCal::Incidence::Status mStatus;
  TQString mParent;

  bool mHasStartDate;

  bool mHasDueDate;
  TQDateTime mDueDate;

  bool mHasCompletedDate;
  TQDateTime mCompletedDate;
};

}

#endif // KOLAB_TASK_H
